@@ -1,4 +1,5 @@
 #![doc = include_str!("../README.md")]
+#![cfg_attr(docsrs, feature(doc_auto_cfg))]
 #![deny(unsafe_code)]
 
 #[cfg(feature = "serde")]
@@ -46,6 +47,8 @@ impl<T: Deref<Target = str>> IriRef<T> {
     ///
     /// This operation keeps internally the `iri` parameter and does not allocate.
     ///
+    /// Use [`parse_unchecked`](Self::parse_unchecked) if you already know the IRI is valid to get faster processing.
+    ///
     /// ```
     /// use oxiri::IriRef;
     ///
@@ -53,13 +56,28 @@ impl<T: Deref<Target = str>> IriRef<T> {
     /// # Result::<(), oxiri::IriParseError>::Ok(())
     /// ```
     pub fn parse(iri: T) -> Result<Self, IriParseError> {
-        let positions = IriParser::parse(&iri, None, &mut VoidOutputBuffer::default())?;
+        let positions = IriParser::<_, false>::parse(&iri, None, &mut VoidOutputBuffer::default())?;
         Ok(Self { iri, positions })
     }
 
+    /// Variant of [`parse`](Self::parse) that assumes that the IRI is valid to skip validation.
+    ///
+    /// ```
+    /// use oxiri::IriRef;
+    ///
+    /// IriRef::parse_unchecked("//foo.com/bar/baz");
+    /// ```
+    pub fn parse_unchecked(iri: T) -> Self {
+        let positions =
+            IriParser::<_, true>::parse(&iri, None, &mut VoidOutputBuffer::default()).unwrap();
+        Self { iri, positions }
+    }
+
     /// Validates and resolved a relative IRI against the current IRI
     /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
+    /// Use [`resolve_unchecked`](Self::resolve_unchecked) if you already know the IRI is valid to get faster processing.
+    ///
     /// ```
     /// use oxiri::IriRef;
     ///
@@ -70,18 +88,39 @@ impl<T: Deref<Target = str>> IriRef<T> {
     /// ```
     pub fn resolve(&self, iri: &str) -> Result<IriRef<String>, IriParseError> {
         let mut target_buffer = String::with_capacity(self.iri.len() + iri.len());
-        let positions = IriParser::parse(iri, Some(self.as_ref()), &mut target_buffer)?;
+        let positions = IriParser::<_, false>::parse(iri, Some(self.as_ref()), &mut target_buffer)?;
         Ok(IriRef {
             iri: target_buffer,
             positions,
         })
     }
 
+    /// Variant of [`resolve`](Self::resolve) that assumes that the IRI is valid to skip validation.
+    ///
+    /// ```
+    /// use oxiri::IriRef;
+    ///
+    /// let base_iri = IriRef::parse_unchecked("//foo.com/bar/baz");
+    /// let iri = base_iri.resolve_unchecked("bat#foo");
+    /// assert_eq!(iri.into_inner(), "//foo.com/bar/bat#foo");
+    /// ```
+    pub fn resolve_unchecked(&self, iri: &str) -> IriRef<String> {
+        let mut target_buffer = String::with_capacity(self.iri.len() + iri.len());
+        let positions =
+            IriParser::<_, true>::parse(iri, Some(self.as_ref()), &mut target_buffer).unwrap();
+        IriRef {
+            iri: target_buffer,
+            positions,
+        }
+    }
+
     /// Validates and resolved a relative IRI against the current IRI
     /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
     /// It outputs the resolved IRI into `target_buffer` to avoid any memory allocation.
     ///
+    /// Use [`resolve_into_unchecked`](Self::resolve_into_unchecked) if you already know the IRI is valid to get faster processing.
+    ///
     /// ```
     /// use oxiri::IriRef;
     ///
@@ -92,10 +131,24 @@ impl<T: Deref<Target = str>> IriRef<T> {
     /// # Result::<(), oxiri::IriParseError>::Ok(())
     /// ```
     pub fn resolve_into(&self, iri: &str, target_buffer: &mut String) -> Result<(), IriParseError> {
-        IriParser::parse(iri, Some(self.as_ref()), target_buffer)?;
+        IriParser::<_, false>::parse(iri, Some(self.as_ref()), target_buffer)?;
         Ok(())
     }
 
+    /// Variant of [`resolve_into`](Self::resolve_into) that assumes that the IRI is valid to skip validation.
+    ///
+    /// ```
+    /// use oxiri::IriRef;
+    ///
+    /// let base_iri = IriRef::parse_unchecked("//foo.com/bar/baz");
+    /// let mut result = String::default();
+    /// let iri = base_iri.resolve_into_unchecked("bat#foo", &mut result);
+    /// assert_eq!(result, "//foo.com/bar/bat#foo");
+    /// ```
+    pub fn resolve_into_unchecked(&self, iri: &str, target_buffer: &mut String) {
+        IriParser::<_, true>::parse(iri, Some(self.as_ref()), target_buffer).unwrap();
+    }
+
     /// Returns an `IriRef` borrowing this IRI's text.
     #[inline]
     pub fn as_ref(&self) -> IriRef<&str> {
@@ -492,6 +545,8 @@ impl<T: Deref<Target = str>> Iri<T> {
     ///
     /// This operation keeps internally the `iri` parameter and does not allocate.
     ///
+    /// Use [`parse_unchecked`](Self::parse_unchecked) if you already know the IRI is valid to get faster processing.
+    ///
     /// ```
     /// use oxiri::Iri;
     ///
@@ -502,9 +557,22 @@ impl<T: Deref<Target = str>> Iri<T> {
         IriRef::parse(iri)?.try_into()
     }
 
+    /// Variant of [`parse`](Self::parse) that assumes that the IRI is valid to skip validation.
+    ///
+    /// ```
+    /// use oxiri::Iri;
+    ///
+    /// Iri::parse_unchecked("http://foo.com/bar/baz");
+    /// ```
+    pub fn parse_unchecked(iri: T) -> Self {
+        Iri(IriRef::parse_unchecked(iri))
+    }
+
     /// Validates and resolved a relative IRI against the current IRI
     /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
+    /// Use [`resolve_unchecked`](Self::resolve_unchecked) if you already know the IRI is valid to get faster processing.
+    ///
     /// ```
     /// use oxiri::Iri;
     ///
@@ -517,11 +585,26 @@ impl<T: Deref<Target = str>> Iri<T> {
         Ok(Iri(self.0.resolve(iri)?))
     }
 
+    /// Variant of [`resolve`](Self::resolve) that assumes that the IRI is valid to skip validation.
+    ///
+    /// ```
+    /// use oxiri::Iri;
+    ///
+    /// let base_iri = Iri::parse_unchecked("http://foo.com/bar/baz");
+    /// let iri = base_iri.resolve_unchecked("bat#foo");
+    /// assert_eq!(iri.into_inner(), "http://foo.com/bar/bat#foo");
+    /// ```
+    pub fn resolve_unchecked(&self, iri: &str) -> Iri<String> {
+        Iri(self.0.resolve_unchecked(iri))
+    }
+
     /// Validates and resolved a relative IRI against the current IRI
     /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
     /// It outputs the resolved IRI into `target_buffer` to avoid any memory allocation.
     ///
+    /// Use [`resolve_into_unchecked`](Self::resolve_into_unchecked) if you already know the IRI is valid to get faster processing.
+    ///
     /// ```
     /// use oxiri::Iri;
     ///
@@ -535,6 +618,20 @@ impl<T: Deref<Target = str>> Iri<T> {
         self.0.resolve_into(iri, target_buffer)
     }
 
+    /// Variant of [`resolve_into`](Self::resolve_into) that assumes that the IRI is valid to skip validation.
+    ///
+    /// ```
+    /// use oxiri::Iri;
+    ///
+    /// let base_iri = Iri::parse_unchecked("http://foo.com/bar/baz");
+    /// let mut result = String::default();
+    /// let iri = base_iri.resolve_into_unchecked("bat#foo", &mut result);
+    /// assert_eq!(result, "http://foo.com/bar/bat#foo");
+    /// ```
+    pub fn resolve_into_unchecked(&self, iri: &str, target_buffer: &mut String) {
+        self.0.resolve_into_unchecked(iri, target_buffer)
+    }
+
     /// Returns an IRI borrowing this IRI's text
     #[inline]
     pub fn as_ref(&self) -> Iri<&str> {
@@ -907,12 +1004,12 @@ impl fmt::Display for IriParseError {
         match &self.kind {
             IriParseErrorKind::NoScheme => write!(f, "No scheme found in an absolute IRI"),
             IriParseErrorKind::InvalidHostCharacter(c) => {
-                write!(f, "Invalid character '{}' in host", c)
+                write!(f, "Invalid character '{c}' in host")
             }
-            IriParseErrorKind::InvalidHostIp(e) => write!(f, "Invalid host IP ({})", e),
-            IriParseErrorKind::InvalidPortCharacter(c) => write!(f, "Invalid character '{}'", c),
+            IriParseErrorKind::InvalidHostIp(e) => write!(f, "Invalid host IP ({e})"),
+            IriParseErrorKind::InvalidPortCharacter(c) => write!(f, "Invalid character '{c}'"),
             IriParseErrorKind::InvalidIriCodePoint(c) => {
-                write!(f, "Invalid IRI code point '{}'", c)
+                write!(f, "Invalid IRI code point '{c}'")
             }
             IriParseErrorKind::InvalidPercentEncoding(cs) => write!(
                 f,
@@ -1038,6 +1135,7 @@ struct ParserInput<'a> {
     value: Chars<'a>,
     position: usize,
 }
+
 impl<'a> ParserInput<'a> {
     #[inline]
     fn next(&mut self) -> Option<char> {
@@ -1063,7 +1161,7 @@ impl<'a> ParserInput<'a> {
 /// parser implementing https://url.spec.whatwg.org/#concept-basic-url-parser without the normalization or backward compatibility bits to comply with RFC 3987
 ///
 /// A sub function takes care of each state
-struct IriParser<'a, O: OutputBuffer> {
+struct IriParser<'a, O: OutputBuffer, const UNCHECKED: bool> {
     iri: &'a str,
     base: Option<IriRef<&'a str>>,
     input: ParserInput<'a>,
@@ -1072,7 +1170,7 @@ struct IriParser<'a, O: OutputBuffer> {
     input_scheme_end: usize,
 }
 
-impl<'a, O: OutputBuffer> IriParser<'a, O> {
+impl<'a, O: OutputBuffer, const UNCHECKED: bool> IriParser<'a, O, UNCHECKED> {
     fn parse(
         iri: &'a str,
         base: Option<IriRef<&'a str>>,
@@ -1100,7 +1198,13 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
 
     fn parse_scheme_start(&mut self) -> Result<(), IriParseError> {
         match self.input.front() {
-            Some(':') => self.parse_error(IriParseErrorKind::NoScheme),
+            Some(':') => {
+                if UNCHECKED {
+                    self.parse_scheme()
+                } else {
+                    self.parse_error(IriParseErrorKind::NoScheme)
+                }
+            }
             Some(c) if c.is_ascii_alphabetic() => self.parse_scheme(),
             _ => self.parse_relative(),
         }
@@ -1260,8 +1364,10 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
                 self.output.push(c);
                 if c == ']' {
                     let ip = &self.iri[start_position + 1..self.input.position - 1];
-                    if let Err(error) = Ipv6Addr::from_str(ip) {
-                        return self.parse_error(IriParseErrorKind::InvalidHostIp(error));
+                    if !UNCHECKED {
+                        if let Err(error) = Ipv6Addr::from_str(ip) {
+                            return self.parse_error(IriParseErrorKind::InvalidHostIp(error));
+                        }
                     }
 
                     let c = self.input.next();
@@ -1274,11 +1380,24 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
                             self.output_positions.authority_end = self.output.len();
                             self.parse_path_start(c)
                         }
-                        Some(c) => self.parse_error(IriParseErrorKind::InvalidHostCharacter(c)),
+                        Some(c) => {
+                            if UNCHECKED {
+                                self.output.push(c);
+                                continue;
+                            } else {
+                                self.parse_error(IriParseErrorKind::InvalidHostCharacter(c))
+                            }
+                        }
                     };
                 }
             }
-            self.parse_error(IriParseErrorKind::InvalidHostCharacter('['))
+            if UNCHECKED {
+                // We consider it's valid even if it's not finished
+                self.output_positions.authority_end = self.output.len();
+                self.parse_path_start(None)
+            } else {
+                self.parse_error(IriParseErrorKind::InvalidHostCharacter('['))
+            }
         } else {
             // Other host
             loop {
@@ -1302,12 +1421,17 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
         loop {
             let c = self.input.next();
             match c {
-                Some(c) if c.is_ascii_digit() => self.output.push(c),
                 Some('/') | Some('?') | Some('#') | None => {
                     self.output_positions.authority_end = self.output.len();
                     return self.parse_path_start(c);
                 }
-                Some(c) => return self.parse_error(IriParseErrorKind::InvalidPortCharacter(c)),
+                Some(c) => {
+                    if UNCHECKED || c.is_ascii_digit() {
+                        self.output.push(c)
+                    } else {
+                        return self.parse_error(IriParseErrorKind::InvalidPortCharacter(c));
+                    }
+                }
             }
         }
     }
@@ -1366,7 +1490,7 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
                         self.output_positions.query_end = self.output.len();
                         self.output.push('#');
                         return self.parse_fragment();
-                    } else if c == None {
+                    } else if c.is_none() {
                         self.output_positions.path_end = self.output.len();
                         self.output_positions.query_end = self.output.len();
                         return Ok(());
@@ -1408,11 +1532,11 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
 
     #[inline]
     fn read_url_codepoint_or_echar(&mut self, c: char) -> Result<(), IriParseError> {
-        if c == '%' {
-            self.read_echar()
-        } else if is_url_code_point(c) {
+        if UNCHECKED || is_url_code_point(c) {
             self.output.push(c);
             Ok(())
+        } else if c == '%' {
+            self.read_echar()
         } else {
             self.parse_error(IriParseErrorKind::InvalidIriCodePoint(c))
         }
@@ -1420,11 +1544,11 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
 
     #[inline]
     fn read_url_query_codepoint_or_echar(&mut self, c: char) -> Result<(), IriParseError> {
-        if c == '%' {
-            self.read_echar()
-        } else if is_url_query_code_point(c) {
+        if UNCHECKED || is_url_query_code_point(c) {
             self.output.push(c);
             Ok(())
+        } else if c == '%' {
+            self.read_echar()
         } else {
             self.parse_error(IriParseErrorKind::InvalidIriCodePoint(c))
         }
@@ -1456,6 +1580,7 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
     }
 }
 
+#[inline]
 fn is_url_code_point(c: char) -> bool {
     matches!(c,
         'a'..='z'
@@ -1500,6 +1625,7 @@ fn is_url_code_point(c: char) -> bool {
     )
 }
 
+#[inline]
 fn is_url_query_code_point(c: char) -> bool {
     is_url_code_point(c)
         || matches!(c, '\u{E000}'..='\u{F8FF}' | '\u{F0000}'..='\u{FFFFD}' | '\u{100000}'..='\u{10FFFD}')
