@@ -37,8 +37,10 @@ fn test_parsing() {
     ];
 
     for e in examples.iter() {
-        if let Err(error) = Iri::parse(*e) {
-            panic!("{} on IRI {}", error, e);
+        let unchecked = Iri::parse_unchecked(*e);
+        match Iri::parse(*e) {
+            Ok(iri) => assert_eq!(iri, unchecked),
+            Err(error) => panic!("{} on IRI {}", error, e),
         }
     }
 }
@@ -116,7 +118,7 @@ fn test_relative_parsing() {
         //-- bare authority
         "http://example.org",
         //-- IPv6 literals (from RFC2732):
-        "http://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:80/index.html",
+        "http://[FEDC:AA98:7654:3210:FEDC:AA98:7654:3210]:80/index.html",
         "http://[1080:0:0:0:8:800:200C:417A]/index.html",
         "http://[3ffe:2a00:100:7031::1]",
         "http://[1080::8:800:200C:417A]/foo",
@@ -134,11 +136,14 @@ fn test_relative_parsing() {
 
     let base = Iri::parse("http://a/b/c/d;p?q").unwrap();
     for e in examples.iter() {
-        if let Err(error) = IriRef::parse(*e) {
-            panic!("{} on relative IRI {}", error, e);
+        let unchecked = IriRef::parse_unchecked(*e);
+        match IriRef::parse(*e) {
+            Ok(iri) => assert_eq!(unchecked, iri),
+            Err(error) => panic!("{} on relative IRI {}", error, e),
         }
-        if let Err(error) = base.resolve(*e) {
-            panic!("{} on relative IRI {}", error, e);
+        match base.resolve(e) {
+            Ok(iri) => assert_eq!(base.resolve_unchecked(e), iri),
+            Err(error) => panic!("{} on relative IRI {}", error, e),
         }
     }
 }
@@ -209,21 +214,21 @@ fn test_wrong_relative_parsing() {
         "?\u{FFFF}",
         "/\u{FDEF}",
         "/\u{FFFF}",
-        "/u{1FFFF}",
-        "/u{2FFFF}",
-        "/u{3FFFF}",
-        "/u{4FFFF}",
-        "/u{5FFFF}",
-        "/u{6FFFF}",
-        "/u{7FFFF}",
-        "/u{8FFFF}",
-        "/u{9FFFF}",
-        "/u{AFFFF}",
-        "/u{BFFFF}",
-        "/u{CFFFF}",
-        "/u{DFFFF}",
-        "/u{EFFFF}",
-        "/u{FFFFF}",
+        "/\u{1FFFF}",
+        "/\u{2FFFF}",
+        "/\u{3FFFF}",
+        "/\u{4FFFF}",
+        "/\u{5FFFF}",
+        "/\u{6FFFF}",
+        "/\u{7FFFF}",
+        "/\u{8FFFF}",
+        "/\u{9FFFF}",
+        "/\u{AFFFF}",
+        "/\u{BFFFF}",
+        "/\u{CFFFF}",
+        "/\u{DFFFF}",
+        "/\u{EFFFF}",
+        "/\u{FFFFF}",
         // bad host
         "http://[/",
         "http://[::1]a/",
@@ -233,7 +238,7 @@ fn test_wrong_relative_parsing() {
 
     let base = Iri::parse("http://a/b/c/d;p?q").unwrap();
     for e in examples.iter() {
-        let result = base.resolve(*e);
+        let result = base.resolve(e);
         assert!(result.is_err(), "{} is wrongly considered valid", e);
     }
 }
@@ -520,21 +525,23 @@ fn test_resolve_relative_iri() {
 
     for (relative, base, output) in examples.iter() {
         let base = Iri::parse(*base).unwrap();
-        match base.resolve(*relative) {
+        match base.resolve(relative) {
             Ok(result) => assert_eq!(
                 result.as_str(),
                 *output,
-                "Resolving of {} against {} is wrong. Found {} and expecting {}",
-                relative,
-                base,
-                result,
-                output
+                "Resolving of {relative} against {base} is wrong. Found {result} and expecting {output}"
             ),
             Err(error) => panic!(
                 "Resolving of {} against {} failed with error: {}",
                 relative, base, error
             ),
         }
+        let result = base.resolve_unchecked(relative);
+        assert_eq!(
+            result.as_str(),
+            *output,
+            "Lenient resolving of {relative} against {base} is wrong. Found {result} and expecting {output}"
+        );
     }
 }
 
